%% Exercice Regression Softmax
%% Source : tutoriel de Stanford :  
%% http://ufldl.stanford.edu/wiki/index.php/Exercise:Softmax_Regression

%  Instructions
%  ------------
% 
%  Ce fichier contient les instructions principales de l'exercice. Il n'est pas a modifier. 
%  Vous devez implementer softmaxCost.m et la prediction par la fonction softmax dans softmaxPred.m. 

%%======================================================================
%% ETAPE 0: Initialise des constantes et parametres
%

tailleInput = 28 * 28; % taille des vecteurs en entree (images MNIST: 28x28)
nbClasses = 10;     % Nombre de classes (images MNIST: 10 classes, chiffres de 0 a 9)

lambda = 1e-4; % parametre de regularisation

%%======================================================================
%% ETAPE 1: charger les donnees
%
% Input : les images
% output : les classes sont les chiffres "labels" ou etiquettes.

images = loadMNISTImages('mnist/train-images-idx3-ubyte');
labels = loadMNISTLabels('mnist/train-labels-idx1-ubyte');
labels(labels==0) = 10; % la classe 10 represente 0, plus facile comme ca car les indices en matlab commencent a 1.

inputData = images;

% Affiche 12 images :   
% rand_indices = randperm(size(images,2));
% sel = images(:,rand_indices(1:12));
% displayData(sel', sqrt(tailleInput));

% fprintf('Program paused. Press enter to continue.\n');
% pause;

% Pour debugger, il vaut mieux utiliser une sous-partie des donnees 
% pour accelerer la verification de votre implementation du gradient.

DEBUG = true; % mettre a false apres avoir verifie que votre calcul de gradient est correct.
if DEBUG
    tailleInput = 8;
    inputData = randn(8, 100);
    labels = randi(10, 100, 1);
    % alternative si randi n'existe pas :
    % labels = round(unifrnd(1,10,100,1))
end

% Initialisation aleatoire de theta
theta = 0.005 * randn(nbClasses * tailleInput, 1);

%%======================================================================
%% ETAPE 2: Implementer softmaxCost
%
%  Implementer softmaxCost dans softmaxCost.m. 

[cost, grad] = softmaxCost(theta, nbClasses, tailleInput, lambda, inputData, labels);
                                     
%%======================================================================
%% ETAPE 3: Verification du gradient
%
%  Comme dans tout algorithme d'apprentissage, il faut toujours verifier
%  votre gradient avant de lancer l'apprentissage de theta.
% 

if DEBUG
    numGrad = computeNumericalGradient( @(x) softmaxCost(x, nbClasses, ...
                                    tailleInput, lambda, inputData, labels), theta);

    % cette ligne sert a comparer les valeurs de votre gradient avec les
    % valeurs attendues :
    disp([numGrad grad]); 

    % Compare le gradient numerique avec votre gradient analytique
    diff = norm(numGrad-grad)/norm(numGrad+grad);
    disp(diff); 
    % Cette difference doit etre petite : moins de 1e-7 
end

%%======================================================================
%% ETAPE 4 : apprentissage des parametres
%
%  Une fois que vous avez verifie que vos gradients sont corrects,
%  vous pouvez lancer la commande suivante. 
%  La fonction softmaxTrain utilise la fonction d'optimisation fournie
%  minFunc.

options.maxIter = 100;
softmaxModel = softmaxTrain(tailleInput, nbClasses, lambda, ...
                            inputData, labels, options);
                          
% Ici, un maximum de 100 iterations sont faites. En pratique, en faire plus est benefique 
% en general

%%======================================================================
%% ETAPE 5: Test
%
%  Vous allez maintenant tester votre modele sur les images de test. 
%  Pour cela, vous devez completer la fonction softmaxPredict.m, qui
%  retourne les predictions etant donne un modele softmax et les donnees de
%  test

images = loadMNISTImages('mnist/t10k-images-idx3-ubyte');
labels = loadMNISTLabels('mnist/t10k-labels-idx1-ubyte');
labels(labels==0) = 10; % 0 -> classe 10

inputData = images;

% Completez softmaxPredict dans softmaxPredict.m
[pred] = softmaxPredict(softmaxModel, inputData);

correct = mean(labels(:) == pred(:));
fprintf('Performance : %0.3f%%\n', correct * 100);

% correct est la proportion d'images correctement classees.
% Apres 100 iterations, le resultat que nous obtenons est :
%
% Performance : 92.200%
%
